package org.ow2.petals.se.pojo.listener;

import java.lang.reflect.Method;
import java.util.logging.Level;

import javax.jbi.messaging.MessagingException;

import org.ow2.petals.commons.exception.ExceptionUtil;
import org.ow2.petals.component.framework.api.exception.FaultException;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.listener.AbstractJBIListener;
import org.ow2.petals.component.framework.process.async.AsyncContext;
import org.ow2.petals.se.pojo.Pojo;
import org.ow2.petals.se.pojo.PojoComponent;

public class JBIListener extends AbstractJBIListener {

    @Override
    public boolean onJBIMessage(final Exchange exchange) {
        return onMessage(exchange, null);
    }

    @Override
    public boolean onAsyncJBIMessage(final Exchange exchange, final AsyncContext asyncContext) {
        if (asyncContext == null) {
            this.getLogger().log(Level.SEVERE,
                    "onAsyncJBIMessage called with null AsyncContext: ignore !");
            return true;
        }
        return onMessage(exchange, asyncContext);
    }

    private boolean onMessage(final Exchange exchange, final AsyncContext asyncContext) {
        boolean ret = true;
        try {
            Pojo pojo = null;
            if (asyncContext != null)
                pojo = ((PojoComponent) this.getComponent()).getPojos().get(
                        asyncContext.getOriginalExchange().getEndpointName());
            else
                pojo = ((PojoComponent) this.getComponent()).getPojos().get(
                        exchange.getEndpointName());

            if (pojo != null) {
                try {
                    // set the JBI listener at this step, as the instance is
                    // dependent to the processor thread
                    this.setJbiListenerOnPojo(pojo);
                    ret = pojo.callOnExchangeMethod(exchange, asyncContext);
                } catch (final FaultException e) {
                    this.getLogger().log(Level.FINE, e.getMessage(), e);
                    exchange.setFault(e);
                } catch (final PEtALSCDKException e) {
                    this.getLogger().log(Level.SEVERE, e.getMessage(), e);
                    if (exchange.isActiveStatus()) {
                        exchange.setError(new MessagingException(ExceptionUtil
                                .getExtendedMessage(e)));
                    }
                } catch (final MessagingException e) {
                    this.getLogger().log(Level.WARNING, e.getMessage(), e);
                    throw e;
                }
            } else {
                this.getLogger().log(
                        Level.SEVERE,
                        "Failed to find the POJO instance matching the endpoint '"
                                + exchange.getEndpointName() + "'");
                exchange.setError(new MessagingException(
                        "Failed to find the POJO instance matching the endpoint '"
                                + exchange.getEndpointName() + "'"));
            }
        } catch (final MessagingException e) {
            if (exchange.isActiveStatus()) {
                exchange.setError(e);
            }
        }

        return ret;
    }

    /**
     * Find a "setJBIListener(AbstractJBIListener jbiListener)" on the POJO and
     * set the AbstractJBIListener on it. The method is not mandatory.
     * 
     * @throws PEtALSComponentSDKException
     *             invocation failed
     */
    private void setJbiListenerOnPojo(final Pojo pojo) throws PEtALSCDKException {

        // First, we set the SU classloader
        final ClassLoader oldClassLoader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(pojo.getClassLoader());

        try {
            final Method setJbiListenerMethod = Pojo.findMethod(pojo.getPojo().getClass(),
                    Pojo.JBI_LISTENER_SETTER, AbstractJBIListener.class);

            if (setJbiListenerMethod != null) {
                try {
                    setJbiListenerMethod.invoke(pojo.getPojo(), this);
                } catch (final Exception e) {
                    throw new PEtALSCDKException("Can not set the JBIListener on the POJO", e);
                }
            }
        } finally {
            // We restore the thread classloader
            Thread.currentThread().setContextClassLoader(oldClassLoader);
        }
    }
}
